/**
 Drowsy Delegate Receiver Program
 * Based on Xsniffer program from Crossbow
 * Rob Miles 2008
 */
using Crossbow.platform.imote2;
using Crossbow.lib.utils;
using Crossbow.lib.utils.tos;
using Crossbow.radio.cc2420;

namespace Crossbow.app
{
    public class XSniffer
    {
        #region Read Accellerometer 

        static byte[] preamble = new byte[] { 0x55, 0x56, 0x57, 0x58 };

        public static bool getXYZ(byte[] packet, out int x, out int y, out int z)
        {
            x = 0; y = 0; z = 0;
            int index = 0;
            int preamblePos=0;
            while (index < packet.Length)
            {
                if (packet[index] == preamble[preamblePos])
                {
                    preamblePos++;
                    index++;
                    if (preamblePos == preamble.Length)
                    {
                        if (packet.Length - index < 6)
                        {
                            return false;
                        }
                        x = packet[index++];
                        x += (packet[index++] << 8);
                        if ( x>32767 ) x = -(65536-x);
                        y = packet[index++];
                        y += (packet[index++] << 8);
                        if (y > 32767) y = -(65536 - y);
                        z = packet[index++];
                        z += (packet[index++] << 8);
                        if (z > 32767) z = -(65536 - z);
                        return true;
                    }
                }
                else
                {
                    preamblePos = 0;
                    index++;
                }
            }
            return false;
        }

        static int x, y, z;
        static float xTotal = 0, yTotal = 0, zTotal = 0;
        static int count = 0;
        static int values = 100;

        public static void doAverage()
        {
            xTotal += x;
            yTotal += y;
            zTotal += z;
            if (++count == values)
            {
                SerialDump.print("X : " + (xTotal / count).ToString() + " Y : " + (yTotal / count).ToString() + " Z " + (zTotal / count).ToString());
                count = 0;
                xTotal = 0;
                yTotal = 0;
                zTotal = 0;
            }
        }

        #endregion

        #region Nod Management

        static int xReference = 22;
        static int yReference = 74;
        static int zReference = 1001;

        static int xTolerance = 250;
        static int yTolerance = 250;
        static int zTolerance = 250;

        static Leds _leds = new Leds();

        public static void checkNod()
        {
            if ((System.Math.Abs(xReference - x) > xTolerance) ||
                 (System.Math.Abs(yReference - y) > yTolerance) ||
                 (System.Math.Abs(zReference - z) > zTolerance))
            {
                _leds.setRGB(LedColor.RED);
            }
            else
            {
                _leds.setRGB(LedColor.GREEN);
            }

        }

        #endregion

        public static void Main()
        {
            // Create radio receiver

            const ushort TOS_UART_ADDR = 0x7E;
            const ushort _rfChannel = (ushort)RadioChannel.Ch11; // channel 11
            const ushort _rfPower = (ushort)RadioPower.M10DBM;  // -10dbm


            SerialDump.print("XSniffer App started"); //Print out the application identifier

            
            TOSRadio _radio = new TOSRadio(_rfChannel, _rfPower, 0xFFFF, 0xFFFF);
            TosMsg     recvMsg;
            byte[] convertedBuf;

            _leds.setRGB(LedColor.BLUE);

            // Read Radio and decode the messages
            while (true)
            {
                if (null!=(recvMsg =_radio.PromiscuousListening (20)))
                {
                    // PC side only recognizes mica2 TOS message format
                    recvMsg.addr = TOS_UART_ADDR;        // put in the UART address
                    convertedBuf = recvMsg.ConvertToMica2Msg();

                    if (getXYZ(convertedBuf, out x, out y, out z))
                    {
                        // Got a set of values
                        _leds.setRGB(LedColor.GREEN);
                        //SerialDump.print("X : " + x.ToString() + " Y : " + y.ToString() + " Z " + z.ToString());
                        //doAverage();
                        checkNod();
                    }
                    else
                    {
                        SerialDump.print("Invalid message");
                        _leds.setRGB(LedColor.PURPLE);
                    }
                }
            }
         }
    }
}
