/**
 * High Frequency Accel sampling application
 * Sample the accel at a high frequency, send out the packet to both radio and uart
 * 
 * @file        XAccel.cs
 * @author      Ning Xu
 * 
 * Copyright (c) 2007 Crossbow Technology, Inc.   All rights reserved.
 * 
 * $Id: XAccel.cs,v 1.14 2007/09/24 19:13:55 nxu Exp $
 */

using Crossbow.platform.imote2;
using Crossbow.radio.cc2420;
using Crossbow.sensor.its400;
using Crossbow.lib.utils;
using Crossbow.lib.utils.tos;

namespace Crossbow.app
{
    public class XAccel
    {
        static byte[] preamble = new byte[] { 0x55, 0x56, 0x57, 0x58 };

        /// <summary>
        /// This demo application samples the accel (X/Y/Z) at 50HZ, send out the readings 
        /// in a packet with 10 samples in each packet
        /// </summary>
        public static void Main()
        {
            const int _packet_id = 0x01;
            const int _amType = 50;
            const int _groupId = 0x7D;
            const int _sensor_board_id = 0x20;
            const int _default_nodeid = 0x01;
            const ushort _rfChannel = (ushort)RadioChannel.Ch11; // channel 11
            const ushort _rfPower = (ushort)RadioPower.M10DBM;  // -10dbm

            int _nodeid = UniqueID.SerialID;
            if (_nodeid == 0xFFFF)
            {
                _nodeid = _default_nodeid;  // set to default if there is not UID in the flash
            }

            int sampling_interval = 20; // sampling interval;
            const int packet_length = 64; //4-byte sensor header, 60-byte accel data stream (10 samples of x/y/z)
            Leds _leds = new Leds();
            Its400 _sensor = new Its400();
            TOSRadio _radio = new TOSRadio(_rfChannel, _rfPower, 0xFFFF, 0xFFFF);
              
            byte[] packet = new byte[packet_length];
    
            ushort accelX, accelY, accelZ; //accel raw values
            
            SerialDump.print("App [XAccel]: started.");     // Print application identifier
            _leds.setRGB(LedColor.PURPLE);

            int index = 0;
            TosSerialDump.type = _amType;
            TosSerialDump.groupId = _groupId;

            packet[index++] = preamble[0];
            packet[index++] = preamble[1];
            packet[index++] = preamble[2];
            packet[index++] = preamble[3];

            int flashCount = 0;
            const int flashLimit = 100;
            const int halfFlash = 50;
                             
            for (; ; )
            {
                if (flashCount < halfFlash)
                    _leds.setRGB(LedColor.PURPLE);
                else
                    _leds.setRGB(LedColor.YELLOW);

                flashCount++;
                if (flashCount > flashLimit)
                    flashCount = 0;

                accelX = _sensor.AccelXRaw ;
                accelY = _sensor.AccelYRaw;
                accelZ = _sensor.AccelZRaw;

                packet[index++] = (byte)(accelX & 0xFF);
                packet[index++] = (byte)((accelX >> 8) & 0xFF);
                packet[index++] = (byte)(accelY & 0xFF);
                packet[index++] = (byte)((accelY >> 8) & 0xFF);
                packet[index++] = (byte)(accelZ & 0xFF);
                packet[index++] = (byte)((accelZ >> 8) & 0xFF);

                if (index == packet_length)
                {
                    if (_radio != null)
                    {
                        // send it to the radio
                        _radio.SetRadioOption(RadioOption.LocalAddress, (ushort)_nodeid);
                        _radio.amType = _amType;
                        _radio.Send((ushort)0xFFFF, (ushort)0xFFFF, packet);
                    }

                    //send it to uart
                    TosSerialDump.printWithTosHeader(packet); 

                    index = 4;
                    
                }
                System.Threading.Thread.Sleep(sampling_interval);
            }
        }  // end of Main()
    }
}