﻿using System;
namespace GPSDecoder
{
    public interface IGPS
    {
        /// <summary>
        /// Current state of the decode 
        /// </summary>
        GPSDecoderState State
        {
            get;
        }

        /// <summary>
        /// Receives the completed update message from the decode machine
        /// The delegate is called upon successful completion of a message decode
        /// </summary>
        GPSUpdate UpdateReceived
        {
            get;
            set;
        }

        /// <summary>
        /// The most recent completely decoded NEMA message
        /// </summary>
        string LastMessageCode
        {
            get;
        }

        /// <summary>
        /// Date and time of the last known good reading.
        /// </summary>
        DateTime LastGoodReadingValue
        {
            get;
        }

        /// <summary>
        /// Get the decoded position information
        /// </summary>
        /// <param name="LatDegrees">degrees latitude</param>
        /// <param name="LatMinutes">minutes latitude</param>
        /// <param name="LatDir">direction of latitude - N or S</param>
        /// <param name="LongDegrees">degrees longitude</param>
        /// <param name="LongMinutes">minutes longitude</param>
        /// <param name="LongDir">direction of longitude - E or W</param>
        /// <param name="MessageNo">ID no of message. Incremented each time a mmessage is received</param>
        /// <returns>true if successful completion</returns>
        bool GetPosition(ref int LatDegrees, ref float LatMinutes, ref char LatDir, ref int LongDegrees, ref float LongMinutes, ref char LongDir, ref int MessageNo);

        /// <summary>
        /// Delivers a character to the decoder. Will result in a call through to the UpdateReceived delegate
        /// if the character completes a message
        /// </summary>
        /// <param name="ch">character receved</param>
        void ReceiveChar(char ch);

        /// <summary>
        /// Resets the state machine and discards any received information
        /// </summary>
        void Reset();
    }
}
