using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace GPSMFEmulator
{
    public partial class GPSEmulateForm : Form
    {
        private GpsEmulatorProgram parentEmulator;

        public GPSEmulateForm(GpsEmulatorProgram inParentEmulator)
        {
            parentEmulator = inParentEmulator;
            InitializeComponent();
        }

        delegate void SetLcdCallback(Bitmap bitmap, Rectangle rect);

        public void UpdateLcd(Bitmap bitmap, Rectangle rect)
        {
            if (this.lcdPictureBox.InvokeRequired)
            {
                SetLcdCallback d = new SetLcdCallback(UpdateLcd);
                this.Invoke(d, new object[] { bitmap, rect });
            }
            else
            {
                this.lcdPictureBox.Image = bitmap;
            }
        }

        delegate void AddSerialTextCallback(string text);

        public void AddSerialText(string text)
        {
            if (serialOutputTextBox.InvokeRequired)
            {
                AddSerialTextCallback a = new AddSerialTextCallback(AddSerialText);
                this.Invoke(a, new object[] { text });
            }
            else
            {
                this.serialOutputTextBox.AppendText(text);
            }
        }

        delegate void AddGPSTextCallback(string text);

        public void AddGPSText(string text)
        {
            if (this.gpsMessageTextBox.InvokeRequired)
            {
                AddGPSTextCallback a = new AddGPSTextCallback(AddGPSText);
                this.Invoke(a, new object[] { text });
            }
            else
            {
                this.gpsMessageTextBox.AppendText(text);
            }
        }
        
        private void sendInputText()
        {
            parentEmulator.SendToCom(comInputTextBox.Text);
            AddGPSText(comInputTextBox.Text);
            comInputTextBox.Text = "";
        }

        private StreamReader reader;
        private bool pushActive = false;

        private void doPushFile()
        {
            pushActive = true;
            while (!reader.EndOfStream && pushActive)
            {
                string message = reader.ReadLine();
                parentEmulator.SendToCom(message);
                AddGPSText(message);
                System.Threading.Thread.Sleep(message.Length * 2);
            }
            pushActive = false;
            reader.Close();
        }

        private void pushFile(StreamReader reader)
        {
            this.reader = reader;
            System.Threading.Thread workerThread =
                new System.Threading.Thread(new System.Threading.ThreadStart(doPushFile));
            workerThread.Start();
        }

        private void sendFile()
        {
            if (pushActive)
            {
                pushActive = false;
                fileSendButton.Text = "File";
            }
            else
            {
                DialogResult result = gpsOpenFileDialog.ShowDialog();
                if (result != DialogResult.OK)
                {
                    return;
                }

                Stream inputStream = null;

                try
                {
                    inputStream = gpsOpenFileDialog.OpenFile();
                }
                catch
                {
                    MessageBox.Show("File " + gpsOpenFileDialog.FileName + " not opened", "GPS file reader");
                    return;
                }

                StreamReader reader = new StreamReader(inputStream);

                fileSendButton.Text = "Abort";
                pushFile(reader);
            }
        }

        private System.IO.Ports.SerialDataReceivedEventHandler receiver;

        private bool comPortOn = false;

        private bool setupSerialPort()
        {
            portSettingsLabel.Text =
                gpsSerialPort.PortName.ToString() + " " +
                gpsSerialPort.BaudRate.ToString() + " " +
                gpsSerialPort.Parity.ToString();
            try
            {
                receiver = new System.IO.Ports.SerialDataReceivedEventHandler(gpsSerialPort_DataReceived);
                gpsSerialPort.Open();
                gpsSerialPort.DataReceived += receiver;
                comPortButton.Text = "Com Port Off";
                comPortOn = true;
                return true;
            }
            catch
            {
                portSettingsLabel.Text = "No Device";
                return false;
            }
        }

        private bool turnOffSerialPort()
        {
            if (gpsSerialPort.IsOpen)
            {
                gpsSerialPort.DataReceived -= receiver;
                gpsSerialPort.Close();
                comPortButton.Text = "Com Port On";
                comPortOn = false;
                return true;
            }
            return false;
        }

        private void gpsSerialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            if (gpsSerialPort.BytesToRead > 0)
            {
                string message = gpsSerialPort.ReadExisting();
                parentEmulator.SendToCom(message);
                AddGPSText(message);
            }
        }

        private void doComPortToggle()
        {
            if (!comPortOn)
            {
                setupSerialPort();
            }
            else {
                turnOffSerialPort();
            }
        }

        private void comSendButton_Click(object sender, EventArgs e)
        {
            sendInputText();
        }

        private void fileSendButton_Click(object sender, EventArgs e)
        {
            sendFile();
        }

        private void comPortButton_Click(object sender, EventArgs e)
        {
            doComPortToggle();
        }
    }
}