using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using Microsoft.SPOT.Emulator.Gpio;
using Microsoft.SPOT.Emulator.Com;
using Microsoft.SPOT.Emulator.Lcd;
using Microsoft.SPOT.Emulator;

namespace GPSMFEmulator
{
    public class GpsEmulatorProgram : Emulator
    {

        GPSEmulateForm form;

        private ComPortToMemoryStream gpsPort;

        private LcdDisplay lcdDisplay;

        public override void SetupComponent()
        {
            base.SetupComponent();
        }

        public override void InitializeComponent()
        {
            base.InitializeComponent();

            // Start the UI in its own thread.
            Thread uiThread = new Thread(StartForm);
            uiThread.SetApartmentState(ApartmentState.STA);
            uiThread.Start();

            lcdDisplay = this.FindComponentById("LCD_Display") as LcdDisplay;
            lcdDisplay.OnDevicePaint += new OnDevicePaintEventHandler(lcdDisplay_OnDevicePaint);

            gpsPort = this.FindComponentById("GPS_Port") as ComPortToMemoryStream;
            Microsoft.SPOT.Emulator.Time.TimingServices.Timer comTimer =
                new Microsoft.SPOT.Emulator.Time.TimingServices.Timer(
                this.Emulator, new TimerCallback(comMonitor), null, 50, 50);
        }

        void lcdDisplay_OnDevicePaint(object sender, OnDevicePaintEventArgs args)
        {
            form.UpdateLcd(args.Bitmap, args.Rectangle);
        }

        public override void UninitializeComponent()
        {
            base.UninitializeComponent();

            // The emulator is stopped. Close the WinForm UI.
            Application.Exit();
        }

        private void comMonitor(object o)
        {
            int i;
            while (true)
            {
                i = gpsPort.StreamOut.ReadByte();
                if (i == -1)
                {
                    break;
                }
                char ch = (char)i;
                form.AddSerialText(ch.ToString());
            }
        }

        public void SendToCom(string text)
        {
            gpsPort.StreamOut.Write(ASCIIEncoding.ASCII.GetBytes(text), 0, text.Length);
        }

        [STAThread]
        private void StartForm()
        {
            // Some initial setup for the WinForm UI
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            form = new GPSEmulateForm(this);

            // Start the WinForm UI. Run() returns when the form is closed.
            Application.Run(form);

            // When the user closes the WinForm UI, stop the emulator.
            Stop();
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main()
        {
            (new GpsEmulatorProgram()).Start();
        }
    }
}