using System;
using System.Text;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Hardware;

using GPSDecoder;

namespace GPSMicroFlashlight
{

    public class FlashlightProgram
    {

        #region GPS reader

        private static IGPS gps;

        private static SerialPort.Configuration config;
        private static SerialPort port;

        private static void monitorGPSData()
        {
            byte[] buffer = new byte[1000];

            while (true)
            {
                int count = port.Read(
                    buffer,         // destination
                    0,              // start position
                    buffer.Length,  // buffer size
                    50              // wait half a second
                );
                if (count > 0)
                {
                    for (int i = 0; i < count; i++)
                    {
#if gpsdump
                        if (buffer[i] == 10)
                        {
                            Debug.Print(result);
                            result = "";
                        }
                        if (buffer[i] >= 32)
                        {
                            result += (char)buffer[i];
                        }
#endif
                        gps.ReceiveChar((char)buffer[i]);
                    }
                }
            }
        }

        #endregion

        #region GPS Display

        private static Font myFont;

        private static Bitmap myBitmap;

        private static void SetupDisplay()
        {
            myFont = Resources.GetFont(Resources.FontResources.NinaB);

            myBitmap = new Bitmap(
                SystemMetrics.ScreenWidth, SystemMetrics.ScreenHeight);
        }

        private static void GPSUpdated(int messageNumber)
        {
            myBitmap.Clear();
            myBitmap.DrawText(
                gps.LastGoodReadingValue.ToString(),           // string
                myFont,                  // font
                Colors.White,            // text color
                10, 10);                  // draw position

            int latDegrees=0, longDegrees=0;
            char latDir=' ', longDir=' ';
            float latMinutes=0, longMinutes=0;
            int messageNo=0;

            if (gps.GetPosition(
                ref latDegrees, ref latMinutes, ref latDir,
                ref longDegrees, ref longMinutes, ref longDir,
                ref messageNo))
            {

                myBitmap.DrawText(
                    "Lat: " + latDegrees.ToString() + " " +
                    latMinutes.ToString() + latDir.ToString(),
                    myFont,
                    Colors.White,
                    10, 30);

                myBitmap.DrawText(
                    "Long: " + longDegrees.ToString() + " " +
                    longMinutes.ToString() + longDir.ToString(),
                    myFont,
                    Colors.White,
                    10, 50);


                myBitmap.DrawText(
                    messageNo.ToString(),
                    myFont,
                    Colors.White,
                    10, 70);

            }

            myBitmap.Flush();
//            Debug.Print("GPS Update : " + gps.LastMessageCode + "  " + messageNumber);
        }

        #endregion

        public static void Main()
        {
            SetupDisplay();

            myBitmap.Clear();
            myBitmap.DrawText(
                "Waiting...",           // string
                myFont,                  // font
                Colors.White,            // text color
                10, 10);                  // draw position
            myBitmap.Flush();

            config = new SerialPort.Configuration(
                SerialPort.Serial.COM1,
                SerialPort.BaudRate.Baud9600,
                false); // no flow control
            port = new SerialPort(config);

            gps = new GPS();
            gps.UpdateReceived += GPSUpdated;

            monitorGPSData();
        }
    }
}
