using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Microsoft.SPOT.Emulator.MFBook
{
    public partial class GPSForm : Form
    {
        MicroFrameworkGPSComPortComponent port;

        private System.IO.Ports.SerialDataReceivedEventHandler receiver;

        private delegate void AppendComText(string text);

        public GPSForm()
        {

        }
        public GPSForm(MicroFrameworkGPSComPortComponent inPort)
        {
            InitializeComponent();
            port = inPort;
            portSettingsLabel.Text =
                gpsSerialPort.PortName.ToString() + " " +
                gpsSerialPort.BaudRate.ToString() + " " +
                gpsSerialPort.Parity.ToString();
            try
            {
                receiver = new System.IO.Ports.SerialDataReceivedEventHandler(gpsSerialPort_DataReceived);
                gpsSerialPort.Open();
            }
            catch
            {
//                MessageBox.Show("GPS Device not available", "GPS com link");
                gpsDeviceButton.Enabled = false;
            }
        }

        public void AppendText(string text)
        {
            outputTextBox.AppendText(text);
        }

        private bool gpsDeviceEnabledValue = false;

        private bool gpsDeviceEnabled
        {
            get
            {
                return gpsDeviceEnabledValue;
            }
            set
            {
                if (value)
                {
                    if (!gpsDeviceEnabledValue)
                    {
                        comSendTimer.Enabled = true;
                        gpsDeviceEnabledValue = true;
                        gpsDeviceButton.Text = "Turn GPS Device Off";
                    }
                }
                else
                {
                    if (gpsDeviceEnabledValue)
                    {
                        comSendTimer.Enabled = false;
                        gpsDeviceEnabledValue = false;
                        gpsDeviceButton.Text = "Turn GPS Device On";
                    }
                }
            }
        }

        private void GPSForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            gpsSerialPort.Close();
            gpsSerialPort.Dispose();
        }

        private void gpsSerialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            if (gpsSerialPort.BytesToRead > 0)
            {
                System.Threading.Monitor.Enter(gpsSerialPort);
                gpsSerialPort.DataReceived -= receiver;
                string message = gpsSerialPort.ReadExisting();
                System.Threading.ThreadStart send = delegate()
                {
                    AppendText(message);
                    port.SendData(ASCIIEncoding.ASCII.GetBytes(message));
                    System.Threading.Monitor.Exit(gpsSerialPort);
                };
                gpsSerialPort.DataReceived += receiver;
                this.Invoke(send);
            }
        }

        private void gpsDeviceButton_Click(object sender, EventArgs e)
        {
            gpsDeviceEnabled = !gpsDeviceEnabled;
        }

        private void sendTimer_Tick(object sender, EventArgs e)
        {
            if (gpsSerialPort.BytesToRead > 0)
            {
                comSendTimer.Enabled = false;
                string message = gpsSerialPort.ReadExisting();
                AppendText(message);
                port.SendData(ASCIIEncoding.ASCII.GetBytes(message));
                System.Threading.Thread.Sleep(message.Length);
                comSendTimer.Enabled = true;
                return;
            }
        }

        private StreamReader reader = null;

        private void fileButton_Click(object sender, EventArgs e)
        {
            DialogResult result = gpsOpenFileDialog.ShowDialog();
            if (result != DialogResult.OK)
            {
                return;
            }

            Stream inputStream = null;

            try {
                inputStream = gpsOpenFileDialog.OpenFile();
            }
            catch 
            {
                MessageBox.Show("File " + gpsOpenFileDialog.FileName + " not opened","GPS file reader");
                return;
            }

            reader = new StreamReader(inputStream);

            fileButton.Enabled = false;
            fileSendTimer.Enabled = true;
        }

        private void fileSendTimer_Tick(object sender, EventArgs e)
        {
            if (reader != null)
            {
                if (!reader.EndOfStream)
                {
                    fileSendTimer.Enabled = false;
                    string message = reader.ReadLine();
                    message += "\r\n";
                    AppendText(message);
                    port.SendData(ASCIIEncoding.ASCII.GetBytes(message));
                    System.Threading.Thread.Sleep(message.Length * 10);
                    fileSendTimer.Enabled = true;
                }
                else
                {
                    fileSendTimer.Enabled = false;
                    reader.Close();
                    reader = null;
                    fileButton.Enabled = true;
                }
            }

        }
   }
}