using System;
using System.Threading;
using System.Windows.Forms;

using System.Collections.Generic;
using System.Text;

using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Emulator;
using Microsoft.SPOT.Emulator.Gpio;

namespace Microsoft.SPOT.Emulator.MFBook
{
    public class LCDPanelComponent : EmulatorComponent
    {
        private Thread uiThread;                            // Thread that handles the UI
        private LCDPanelForm myForm;                       // Handle to the UI form

        private delegate void CloseLCD_UIDelegate();        // Delegate called when the UI is being closed

        /// <summary>
        /// Constructor of this sample.
        /// </summary>
        public LCDPanelComponent()
        {
            // Instantiate a new UI form
            myForm = new LCDPanelForm(this);
        }

        /// <summary>
        /// Called by the emulator after the configuration file has been read
        /// </summary>
        public override void SetupComponent()
        {
            // Changing the size of the form.
            // Need to take care of top bar and borders
            myForm.ClientSize = new System.Drawing.Size(this.Emulator.LcdDisplay.Width, this.Emulator.LcdDisplay.Height);

            // Changing the size of the picture container
            myForm.FitImage();

            // Register the handler for the "form disposed" event
            myForm.Disposed += new EventHandler(myForm_Disposed);
        }

        /// <summary>
        /// Called by the emulator after all components were setup and registered
        /// </summary>
        public override void InitializeComponent()
        {
            // Each time the device does an operation on the LCD, this OnDevicePaint will be called
            Emulator.LcdDisplay.OnDevicePaint += new Microsoft.SPOT.Emulator.Lcd.OnDevicePaintEventHandler(LcdDisplay_OnDevicePaint);
                
            // Launching the UI thread.  The UI should be visible now
            uiThread = new Thread(startForm);
            uiThread.Start();
        }

        void LcdDisplay_OnDevicePaint(object sender, Microsoft.SPOT.Emulator.Lcd.OnDevicePaintEventArgs args)
        {
            // Updating the bmp on the form
            myForm.UpdateDisplay(args.Bitmap);
        }

        /// <summary>
        /// Called by the emulator after the program exits
        /// </summary>
        public override void UninitializeComponent()
        {
            if (myForm.InvokeRequired)
            {
                // This gets called when the caller of the method is not running in the UI thread.
                myForm.Invoke(new CloseLCD_UIDelegate(myForm.Close));
            }
            else
            {
                myForm.Close();
            }
        }

        /// <summary>
        /// Called when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void myForm_Disposed(object sender, EventArgs e)
        {
            // Shutting down the emulator.  This will cause all the other components to stop.
            if (Emulator.IsShuttingDown == false)
                Emulator.Stop();
        }

        /// <summary>
        /// Calling the .Net method that will handle all the UI magic
        /// </summary>
        private void startForm()
        {
            Application.Run(myForm);
        }

        /// <summary>
        /// Called each time the device writes to the LCD
        /// </summary>
        /// <param name="bmp">Picture of the LCD</param>
        void LcdDisplay_OnDevicePaint(System.Drawing.Bitmap bmp)
        {
        }
    }
}
