using System;
using System.Threading;
using System.Windows.Forms;

using System.Collections.Generic;
using System.Text;

using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Emulator;
using Microsoft.SPOT.Emulator.Gpio;

namespace Microsoft.SPOT.Emulator.MFBook
{

    public class LampOnGPIO1Component : EmulatorComponent   
    {
        private LampForm ledForm;
        private Thread uiThread;

        private delegate void ChangeState_UIDelegate(bool state);

        /// <summary>
        /// This method is called by the emulator when creating this emulator component
        /// </summary>
        public LampOnGPIO1Component()
        {
            ledForm = new LampForm(this);
        }

        /// <summary>
        /// Called by the emulator after the configuration file has been read
        /// </summary>
        public override void SetupComponent()
        {
            ledForm.Disposed += new EventHandler(myForm_Disposed);
        }

        /// <summary>
        /// Called by the emulator after all components were setup and registered
        /// </summary>
        public override void InitializeComponent()
        {
            Emulator.GpioPorts[(Microsoft.SPOT.Hardware.Cpu.Pin)1].OnGpioActivity +=
                new GpioActivity(GLEDonGPIO1Component_OnGpioActivity);

            // Instantiating the UI thread
            uiThread = new Thread(startForm);

            // UI should appear here.
            uiThread.Start();
        }

        /// <summary>
        /// UI thread calls this .Net method to run the UI form
        /// </summary>
        private void startForm()
        {
            Application.Run(ledForm);
        }

        private void GLEDonGPIO1Component_OnGpioActivity(GpioPort port, bool state)
        {
            if (port.Pin != (Cpu.Pin)1)
            {
                return;
            }
            if (ledForm.InvokeRequired)
            {
                // This gets called when the caller of the method is not running in the UI thread.
                ledForm.Invoke(new ChangeState_UIDelegate(ledForm.SetState), state);
            }
            else
            {
                ledForm.SetState(state);
            }
        }

        /// <summary>
        /// Called when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void myForm_Disposed(object sender, EventArgs e)
        {
            if (Emulator.IsShuttingDown == false)
                Emulator.Stop();
        }
    }
}
