// Copyright (C) Microsoft Corporation.  All rights reserved.

using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using System.Net.Sockets;
using System.Text;
using System.Net;
using Socket = System.Net.Sockets.Socket;

public static class BlogAlerter
{
    public static void flasher()
    {
        setupOutputs();

        while (true)
        {
            randomDisplay(400, 10);
            if (alert)
            {
                flashRed(600, 20);
                alert = false;
            }

        }
    }

    public static bool alert = false;

    public static System.Threading.Thread flashThread;

    public static void Main()
    {
        flashThread = new System.Threading.Thread(flasher);

        flashThread.Start();

        string address = "www.robmiles.com";
        string name = "/journal/rss.xml";

        String html = GetWebPage(address, name);

        DateTime lastUpdate = getRecentPubDate(html, "<pubDate>");

        Debug.Print("Initial Update Value : " + lastUpdate.ToString());

        while (true)
        {
            System.Threading.Thread.Sleep(10000);
            
            html = GetWebPage(address, name);

            DateTime blogUpdate = getRecentPubDate(html, "<pubDate>");

            if (!blogUpdate.Equals(lastUpdate))
            {
                Debug.Print("Updated at : " + blogUpdate.ToString());
                lastUpdate = blogUpdate;
                alert = true;
            }
        }

    }

    private static void randomDisplay(int speed, int duration)
    {
        for (int i = 0; i < duration; i++)
        {
            displayByte((byte) (Microsoft.SPOT.Math.Random(256)));
            System.Threading.Thread.Sleep(speed);
        }
    }

    private static void flashRed(int speed, int duration)
    {
        for (int i = 0; i < duration; i++)
        {
            displayByte(0x88);
            System.Threading.Thread.Sleep(speed);
            displayByte(0x00);
            System.Threading.Thread.Sleep(speed);
        }
    }


    #region Light Driver

    const byte green = 0x11;
    const byte blue = 0x22;
    const byte yellow = 0x44;
    const byte red = 0x88;

    const Cpu.Pin clockPin = Cpu.Pin.GPIO_Pin0;
    const Cpu.Pin dataPin = Cpu.Pin.GPIO_Pin1;
    const Cpu.Pin latchPin = Cpu.Pin.GPIO_Pin2;

    static OutputPort clockPort;
    static OutputPort dataPort;
    static OutputPort latchPort;


    private static void setupOutputs()
    {
        clockPort = new OutputPort(clockPin, false);
        dataPort = new OutputPort(dataPin, false);
        latchPort = new OutputPort(latchPin, false);
    }

    private static void displayByte(byte value)
    {
        latchPort.Write(false);
        clockPort.Write(false);

        byte mask = 1;

        for (int i = 0; i < 8; i++)
        {
            if ((value & mask) > 0)
            {
                dataPort.Write(true);
            }
            else
            {
                dataPort.Write(false);
            }
            clockPort.Write(true);
            clockPort.Write(false);
            mask <<= 1;
        }
        latchPort.Write(true);
        latchPort.Write(false);
    }


    public static void testPort(OutputPort port, string name)
    {
        Debug.Print("Testing: " + name);
        port.Write(false);
        System.Threading.Thread.Sleep(1000);
        port.Write(true);
        System.Threading.Thread.Sleep(1000);
        port.Write(false);
        System.Threading.Thread.Sleep(1000);
    }

    public static void testOutputPorts()
    {
        while (true)
        {
            testPort(clockPort, "Clock");
            testPort(dataPort, "Data");
            testPort(latchPort, "Latch");
        }
    }


    #endregion


    #region Date Decode

    private static bool isDigit(char ch)
    {
        return "0123456789".IndexOf(ch) >= 0;
    }

    private static readonly string[] MonthNames = new string[]
        {"Jan", "Feb", "Mar", "Apr", "May", "Jun", 
         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

    private static int getMonth(string monthText)
    {
        for (int i = 0; i < 12; i++)
        {
            if (monthText == MonthNames[i]) return i + 1;
        }
        throw new Exception("Invalid month name : " + monthText);
    }

    private static string getString(string input, ref int startPos, char termCh)
    {
        // spin to first digit
        while (startPos < input.Length && input[startPos]==' ') startPos++;

        // abandon if we hit the end
        if (startPos == input.Length) throw new Exception("Short String");

        string result="";

        while ( input[startPos] != termCh) 
        {
            result = result + input[startPos];
            startPos++;
        }

        return result;
    }

    private static int getInt(string input, ref int startPos, char termCh)
    {
        // spin to first digit
        while (startPos < input.Length && !isDigit(input[startPos])) startPos++;

        // abandon if we hit the end
        if (startPos == input.Length) throw new Exception("Short Integer");

        int result = 0;

        // spin to non digit
        while (startPos < input.Length && isDigit(input[startPos])) 
        {
            result = 10 * result + (int)(input[startPos] - '0');
            startPos++;
        }
        return result;
    }


    /// <summary>
    /// Gets the most recent publication date from the RSS feed by evil scraping
    /// 
    /// 
    /// <pubDate>Wed, 05 Nov 2008 22:38:52 +0000</pubDate>
    /// 
    /// 
    /// </summary>
    /// <param name="html"></param>
    /// <returns></returns>
    private static DateTime getRecentPubDate(string html, string startTag)
    {
        int index = html.IndexOf(startTag) ;

        if (index + startTag.Length > html.Length) throw new Exception("Short string");

        index += startTag.Length;

        // spin past the name of the day
        while (index < html.Length && html[index] != ',') index++;
        if (index == html.Length) throw new Exception("Short publish date");

        int dayValue = getInt(html, ref index, ' ');

        string monthName = getString(html, ref index, ' ');
        int monthValue = getMonth(monthName);

        int yearValue = getInt(html, ref index, ' ');
        int hourValue = getInt(html, ref index, ':');
        int minuteValue = getInt(html, ref index, ':');
        int secondValue = getInt(html, ref index, ' ');

        return new DateTime(yearValue, monthValue, dayValue, hourValue, minuteValue, secondValue);
    }

    #endregion

    #region Web Page Fetch

    // This method requests the home page content for the specified server.
    private static String GetWebPage(String server, string webPage)
    {
        const Int32 c_httpPort = 80;
        const Int32 c_microsecondsPerSecond = 1000000;

        // Create a socket connection to the specified server and port.
        using (Socket serverSocket = ConnectSocket(server, c_httpPort))
        {
            // Send request to the server.
            String request = "GET "+webPage+ " HTTP/1.1\r\nHost: " + server + "\r\nConnection: Close\r\n\r\n";
            Byte[] bytesToSend = Encoding.UTF8.GetBytes(request);
            serverSocket.Send(bytesToSend, bytesToSend.Length, 0);

            // Allocate a buffer that we'll keep reusing to receive HTML chunks
            Byte[] buffer = new Byte[1024];

            // 'page' refers to the HTML data as it is built up.
            String page = String.Empty; 

            // Wait up to 30 seconds for initial data to be available
            // Will throw exception if connection closed with no data sent
            DateTime timeoutAt = DateTime.Now.AddSeconds(30);
            while (serverSocket.Available == 0 && DateTime.Now < timeoutAt)
            {
                System.Threading.Thread.Sleep(100);
            }

            // Poll for data until 30 second time out - Returns true for data and connection closed
            while (serverSocket.Poll(30 * c_microsecondsPerSecond, SelectMode.SelectRead))
            {
                // Zero all bytes in the re-usable buffer
                Array.Clear(buffer, 0, buffer.Length);

                // Read a buffer-sized HTML chunk
                Int32 bytesRead = serverSocket.Receive(buffer);

                // If 0 bytes in buffer, then connection is closed, or we have timed out
                if (bytesRead == 0)
                    break;

                // Append the chunk to the string
                page = page + new String(Encoding.UTF8.GetChars(buffer));
            }

            return page;   // Return the complete string
        }
    }

    private static Socket ConnectSocket(String server, Int32 port)
    {
        // Get server's IP address.
        IPHostEntry hostEntry = Dns.GetHostEntry(server);

        // Create socket and connect to the server's IP address and port
        Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
        socket.Connect(new IPEndPoint(hostEntry.AddressList[0], port));
        return socket;
    }

    #endregion

}
